@@ -1,6 +1,5 @@
-import datetime
-
 import rframe
+import datetime
 
 from .base_schemas import VersionedXeDoc
 
@@ -13,10 +12,10 @@ class Bodega(VersionedXeDoc):
 
     _ALIAS = "bodega"
 
-    field: str = rframe.Index()
+    field: str = rframe.Index(max_length=50)
 
     value: float
     uncertainty: float
     definition: str
-    reference: str
+    reference: str = ''
     date: datetime.datetime
