@@ -1,6 +1,6 @@
 
 import datetime
-
+import pydantic
 import rframe
 from pydantic import validator
 from typing import Literal
@@ -8,28 +8,24 @@ from typing import Literal
 from ..base_schemas import XeDoc
 from ..._settings import settings
 
-SOURCE_TYPE = Literal['ambe',
- 'ar-37',
- 'kr-83m',
- 'led',
- 'noise',
- 'none',
- 'rn-220',
- 'th-232',]
+from ..constants import SOURCE
 
 class BaseCalibration(XeDoc):
     """Base class for calibration metadata
     """
     _ALIAS = ""
+    _CATEGORY = "calibration"
 
     class Config:
         allow_population_by_field_name = True
 
     time: rframe.Interval[datetime.datetime] = rframe.IntervalIndex(alias='run_id')
-    source: SOURCE_TYPE
+    source_id: str = rframe.Index(min_length=1, max_length=60)
 
-    operator: str
-    comments: str
+    source_type: SOURCE
+
+    operator: str = pydantic.Field(min_length=1, max_length=60)
+    comments: str = ''
 
     @validator('time', pre=True)
     def run_id_to_time(cls, v):
@@ -40,5 +36,3 @@ class BaseCalibration(XeDoc):
             except:
                 pass
         return v
-
-
