@@ -3,7 +3,7 @@ from typing import Literal
 import rframe
 
 from .base_corrections import TimeSampledCorrection
-
+from ..constants import DETECTOR
 
 class PmtGain(TimeSampledCorrection):
     _ALIAS = "pmt_gains"
@@ -14,7 +14,7 @@ class PmtGain(TimeSampledCorrection):
     # selections on the pmt number. Since this is a index field
     # versioning will be indepentent for each pmt
 
-    detector: Literal["tpc", "neutron_veto", "muon_veto"] = rframe.Index()
-    pmt: int = rframe.Index()
+    detector: DETECTOR = rframe.Index()
+    pmt: int = rframe.Index(ge=0)
 
     value: float
