@@ -11,9 +11,16 @@ import panel as pn
 
 import param
 
-from typing import Any, ClassVar, List, Type
+from typing import Any, ClassVar, List, Literal, Type
 
-from panel.widgets import (CompositeWidget, 
+try:
+    from typing import _LiteralGenericAlias
+except ImportError:
+    _LiteralGenericAlias = None
+
+from panel.widgets import (Widget,
+                           LiteralInput,
+                           CompositeWidget, 
                            DatetimeRangePicker, 
                            EditableRangeSlider)
 
@@ -23,7 +30,9 @@ from tornado.ioloop import IOLoop
 
 from pydantic_panel import (infer_widget, 
                             json_serializable, 
-                            PydanticModelEditor)
+                            PydanticModelEditor,
+                            ItemListEditor)
+
 from pydantic_panel.dispatchers import clean_kwargs
 
 from panel.widgets.slider import _SliderBase
@@ -46,6 +55,15 @@ def json_serializable(value: rframe.Interval):
     return value.left, value.right
 
 
+class NullableInput(LiteralInput):
+    def _process_property_change(self, msg):
+        # msg = Widget._process_property_change(self, msg)
+        if 'value' in msg and msg['value'] == '':
+            msg['value'] = 'null' if self.serializer == 'json' else 'None' 
+
+        msg = super()._process_property_change(msg)
+        return msg
+
 class TimeIntervalEditor(DatetimeRangePicker):
     value = param.ClassSelector(rframe.TimeInterval, default=None)
     
@@ -344,8 +362,24 @@ class QueryEditor(CompositeWidget):
         alias = field.alias if self.by_alias else field_name
 
         alias = alias.replace('_', ' ').capitalize()
-        widget = pn.widgets.LiteralInput(value=value,
-                                            name=alias,)
+        if type(field.outer_type_) == _LiteralGenericAlias:
+            options = list(field.outer_type_.__args__)
+            if value not in options:
+                value = options[0]
+            widget = pn.widgets.MultiChoice(name=alias + 's', 
+                                   value=[value] if value else [], 
+                                   options=options)
+        else:
+            try:
+                widget = ItemListEditor(value=[value] if value else [], 
+                                                class_=field.outer_type_, 
+                                                item_field=field,
+                                                name=alias+ 's',
+                                                )
+            except:
+                widget = NullableInput(value=value,
+                                    name=alias,)
+
         widget.param.watch(self._validate_field, 'value')
         return widget
 
@@ -374,9 +408,13 @@ class QueryEditor(CompositeWidget):
             return
 
         index = self.class_.index_for(name)
+        
         label = index.validate_label(event.new)
 
-        self.value[name] = label
+        if label:
+            self.value[name] = label
+        else:
+            self.value[name] = None
 
         self.param.trigger('value')
 
@@ -386,9 +424,17 @@ class XeDocListEditor(CompositeWidget):
     _composite_type: ClassVar[Type[ListPanel]] = Column
 
     row_size = param.Integer(30)
+    MAX_DISPLAY_ROWS = 20
+    MAX_SEND_ROWS = 500
     column_size = param.Integer(100)
+
+    page_size = param.Integer(15)
+
     class_ = param.ClassSelector(XeDoc, is_instance=False)
-    
+
+    pagination = param.ObjectSelector(default=None, allow_None=True,
+                                      objects=['local', 'remote'])
+
     value = param.List(default=[], class_=XeDoc)
     
     table_widget = param.Parameter(default=None)
@@ -399,8 +445,15 @@ class XeDocListEditor(CompositeWidget):
             self.class_ = self.value[0].__class__
         self._composite[:] = [pn.panel(self.table_view)]
         
-    @param.depends('class_', watch=True, on_init=True)
+    @param.depends('class_', 'pagination', 'page_size', watch=True, on_init=True)
     def _class_changed(self):
+
+        if len(self.value) > self.MAX_SEND_ROWS:
+            self.pagination = 'remote'
+
+        elif len(self.value) > self.MAX_DISPLAY_ROWS:
+            self.pagination = 'local'
+
         docs = [json_serializable(doc.index_labels)
                 for doc in self.value]
         
@@ -415,8 +468,7 @@ class XeDocListEditor(CompositeWidget):
             nwidgets =  2
             ncols = 3
             
-        nrows = max(len(self.value), nwidgets) + 1
-        
+        nrows = max(self.MAX_DISPLAY_ROWS, nwidgets) + 1
         
         self.table_widget = pn.widgets.Tabulator(df,
                                      name="Data (Click to edit)",
@@ -428,20 +480,31 @@ class XeDocListEditor(CompositeWidget):
                                      width_policy='max',
                                      height_policy='max',
                                      embed_content=False,
+                                     page_size=self.page_size,
+                                     pagination=self.pagination,
                                      show_index=False,
                                      )
         
-        
-        
-    @param.depends('value', watch=True)
+    @param.depends('value', watch=True, )
     def _update_table(self):
+        if not self.value:
+            return
+
+        if len(self.value) > self.MAX_SEND_ROWS:
+            self.pagination = 'remote'
+
+        elif len(self.value) > self.MAX_DISPLAY_ROWS:
+            self.pagination = 'local'
+
         if self.table_widget is None:
             return
+
         assert all([isinstance(doc, self.class_) 
                     for doc in self.value]), f"Value must be a list of {self.class_} instances"
 
         docs = [json_serializable(doc.index_labels)
                 for doc in self.value]
+
         df = pd.DataFrame(docs, columns=list(self.class_.get_index_fields()))
         self.table_widget.value = df
     
@@ -512,6 +575,12 @@ class ModelTableEditor(pn.viewable.Viewer):
                                     height=50, width=50, )
         self.dec_page.on_click(self.decrement_page)
 
+        self.find_button = pn.widgets.Button(name='🔍 Query',
+                                        button_type='primary', 
+                                        align='center')
+
+        self.find_button.on_click(self.filter_callback)
+
         self.param.watch(self._update_docs, 
                          ['class_', 'page', 'query', 'refresh_table'])
 
@@ -527,6 +596,7 @@ class ModelTableEditor(pn.viewable.Viewer):
                                      sizing_mode='stretch_both', 
                                      embed_content=False,
                                      show_index=False,
+                                     
                                     #  width=1000,
                                      min_height=500)
 
@@ -552,16 +622,18 @@ class ModelTableEditor(pn.viewable.Viewer):
         self.inc_page.disabled = self.page >= self.last_page or self.page==-1
 
     def filter_callback(self, event):
-        query = self.query_editor.value
+        query = { k:v for k,v in self.query_editor.value.items() if v }
+        query = { k: v[0] if len(v)==1 else v for k,v in query.items() }
+
         if self.query != query:
-            self.query = query                
+            self.query = query               
             self.page = 0
         if self.page == -1:
             self.page = 0
         end = math.ceil(self.class_.compile_query(**self.query).count()/self.page_size)
         self.param.page.bounds = (0, end)
         self.last_page = end
-        self.refresh_table = True
+        self._update_docs()
         
     def trigger_refresh_cb(self, event):
         self.refresh_table = True
@@ -574,6 +646,10 @@ class ModelTableEditor(pn.viewable.Viewer):
 
     def _set_docs(self, future):
         docs = future.result()
+
+        if docs == self.docs:
+            self.table.loading = False
+
         with unlocked():
             self.docs = docs
             docs = [json_serializable(doc.index_labels) for doc in self.docs]
@@ -637,25 +713,20 @@ class ModelTableEditor(pn.viewable.Viewer):
         return pn.Row(self.dec_page, page_view,
                         self.inc_page , max_width=600, width_policy='max')
 
+    @pn.depends('query_editor')
+    def query_panel(self):
+        if self.query_editor is None:
+            return pn.Column()
+        return pn.Column(self.query_editor)
 
-    @pn.depends('class_', 'query', 'page_size')
     def controls_panel(self):
-        
-        find_button = pn.widgets.Button(name='🚀 Query',
-                                        button_type='primary', 
-                                        align='center')
-
-        find_button.on_click(self.filter_callback)
-
-        
         query_container = pn.Card(
-            "Valid JSON or python literals", self.query_editor,
+            "Valid JSON or python literals", self.query_panel,
             header=pn.Row('🔍 Filter Documents', pn.layout.Spacer(), width=300),
             collapsed=True,
         )
-        
 
-        add_new = pn.Card(self.model_editor, 
+        add_new = pn.Card(self.new_doc_panel, 
                           name='insert_new', 
                           header=pn.Row('➕ New Document', 
                                         pn.layout.Spacer(), 
@@ -665,11 +736,18 @@ class ModelTableEditor(pn.viewable.Viewer):
         return pn.Column(
                         
                         query_container,
-                        find_button,
+                        self.find_button,
                         pn.layout.Divider(),
                         add_new,
                         )
-    
+
+    @pn.depends('model_editor')
+    def new_doc_panel(self):
+        if self.model_editor is None:
+            return pn.Column()
+        return pn.Column(self.model_editor,
+                         width_policy='min')
+
     def __panel__(self):
         right_panel = pn.Column(self.page_controls,
                                 self.table_panel)
@@ -742,11 +820,28 @@ class XedocsEditor(pn.viewable.Viewer):
                                      self.param.collection)
 
     @pn.depends('editor')
-    def query_panel(self):
+    def query_or_insert_panel(self):
         if self.editor is None:
             return pn.Column()
         return pn.panel(self.editor.controls_panel)
 
+    @pn.depends('editor')
+    def query_panel(self):
+        if self.editor is None:
+            return pn.Column()
+        
+        return pn.Column(
+                        "### Filters (python literals)",
+                        self.editor.query_panel, 
+                        
+                        width_policy='max')
+
+    @pn.depends('editor')
+    def new_doc_panel(self):
+        if self.editor is None:
+            return pn.Column()
+        return pn.panel(self.editor.new_doc_panel)
+
     @pn.depends('editor')
     def data_panel(self):
         if self.editor is None:
@@ -754,8 +849,16 @@ class XedocsEditor(pn.viewable.Viewer):
         return pn.Column(self.editor.page_controls,
                   self.editor.table_panel)
 
+    @pn.depends('editor')
+    def find_button_panel(self):
+        if self.editor is None:
+            return pn.Row(pn.layout.Spacer(width_policy='max'))
+        return pn.Row(pn.layout.Spacer(width_policy='max'), 
+                              self.editor.find_button, 
+                              pn.layout.Spacer(width_policy='max'))
+
     def __panel__(self):
-        bottom_panel = pn.Row(self.query_panel, self.data_panel)
+        bottom_panel = pn.Row(self.query_or_insert_panel, self.data_panel)
         return pn.Column(self.selection_panel,
                         pn.layout.Divider(),
                         bottom_panel
